@@ -84,6 +84,13 @@ ControlWidget::ControlWidget(const QString &ruleFile, Control *parent)
     connect(ui->cbSimInserted, SIGNAL(clicked()), this, SLOT(simInsertRemove()));
     connect(ui->pbStart, SIGNAL(clicked()), this, SLOT(simAppStart()));
     connect(ui->pbAbort, SIGNAL(clicked()), this, SLOT(simAppAbort()));
+    connect(ui->pbReset, SIGNAL(clicked()), this, SLOT(modemSilentReset()));
+    connect(ui->pbSendGNSSData, SIGNAL(clicked()), this, SLOT(sendGNSSData()));
+    connect(ui->pbGNSSDefault, SIGNAL(clicked()), this, SLOT(setDefaultGNSSData()));
+    connect(ui->pbSendNotif, SIGNAL(clicked()), this, SLOT(sendCSSN()));
+    connect(ui->pbAlerting, SIGNAL(clicked()), this, SLOT(setStateAlerting()));
+    connect(ui->pbActive, SIGNAL(clicked()), this, SLOT(setStateConnected()));
+    connect(ui->pbHangup, SIGNAL(clicked()), this, SLOT(setStateHangup()));
 
     QStringList headers;
     headers << "Sender" << "Priority" << "Notification Status";
@@ -91,6 +98,7 @@ ControlWidget::ControlWidget(const QString &ruleFile, Control *parent)
     ui->twMessageList->verticalHeader()->hide();
 
     handleNewApp();
+    handleCSSNNotif();
 
     show();
 }
@@ -118,7 +126,10 @@ Control::Control(const QString& ruleFile, SimRules *sr, QObject *parent)
         << SIGNAL(command(QString))
         << SIGNAL(variableChanged(QString,QString))
         << SIGNAL(switchTo(QString))
-        << SIGNAL(startIncomingCall(QString));
+        << SIGNAL(startIncomingCall(QString, QString, QString))
+        << SIGNAL(stateChangedToAlerting())
+        << SIGNAL(stateChangedToConnected())
+        << SIGNAL(stateChangedToHangup(int));
 
     foreach (QByteArray sig, proxySignals)
         connect(widget, sig, this, sig);
@@ -129,6 +140,99 @@ Control::~Control()
     delete widget;
 }
 
+void Control::callManagement( QList<CallInfo> *list )
+{
+    int row = 0;
+    bool enableCSSU = false;
+    bool enableCSSI = false;
+    static QString state_table[] = { tr("Active"), tr("Held"),
+                                      tr("Dialing"), tr("Alerting"),
+                                      tr("Incoming"), tr("Waiting"),
+                                      tr("Disconnected"), tr("Swapping") };
+
+    widget->clearCallView();
+
+    foreach( CallInfo i, *list ) {
+        QString param[5];
+
+        if ( i.incoming && !enableCSSU )
+            enableCSSU = true;
+        if ( !i.incoming && !enableCSSI )
+            enableCSSI = true;
+
+        param[0].setNum( i.id );
+        param[1] = i.number;
+        param[2] = state_table[i.state];
+        param[3] = i.name;
+        param[4] = i.incoming ? tr("Incoming") : tr("Outgoing");
+
+        widget->updateCallView( param, row );
+        row++;
+    }
+
+    widget->setCssiEnabled( enableCSSI );
+    widget->setCssuEnabled( enableCSSU );
+}
+
+void ControlWidget::setStateConnected()
+{
+    emit stateChangedToConnected();
+}
+
+void ControlWidget::setStateAlerting()
+{
+    emit stateChangedToAlerting();
+}
+
+void ControlWidget::setStateHangup()
+{
+    QList <QTableWidgetItem *> items = ui->twCallMgt->selectedItems();
+
+    foreach ( QTableWidgetItem *item, items )
+    {
+        int row = item->row();
+        if ( row >= 0 )
+        {
+            QTableWidgetItem *itemCallId = ui->twCallMgt->item( row, 0 );
+            if ( itemCallId != 0 )
+            {
+                QString strid = itemCallId->text();
+                int id = strid.toInt();
+                emit stateChangedToHangup( id );
+            }
+        }
+    }
+}
+
+void ControlWidget::setCssiEnabled( bool enableCSSI )
+{
+    ui->cbCSSI->setEnabled( enableCSSI );
+}
+
+void ControlWidget::setCssuEnabled( bool enableCSSU )
+{
+    ui->cbCSSU->setEnabled( enableCSSU );
+}
+
+void ControlWidget::clearCallView()
+{
+    ui->twCallMgt->setRowCount(0);
+    ui->twCallMgt->clearContents();
+}
+
+void ControlWidget::updateCallView( QString callParameters [5], int row )
+{
+    if ( row > ui->twCallMgt->rowCount() - 1 )
+        ui->twCallMgt->insertRow( row );
+
+    for ( int i = 0; i < 5; i++ )
+    {
+        QTableWidgetItem *item = new QTableWidgetItem( callParameters[i] );
+        item->setFlags( item->flags() & ~Qt::ItemIsEditable );
+        ui->twCallMgt->setItem( row, i, item );
+    }
+}
+
 void Control::setPhoneNumber( const QString &number )
 {
     widget->setWindowTitle("Phonesim - Number: " + number);
@@ -139,13 +243,42 @@ void Control::warning( const QString &title, const QString &message )
     QMessageBox::warning(widget, title, message, "OK");
 }
 
+void ControlWidget::handleCSSNNotif()
+{
+    ui->cbCSSU->setEnabled( false );
+    ui->cbCSSI->setEnabled( false );
+
+    ui->cbCSSU->insertItem( 0, "" );
+    ui->cbCSSU->insertItem( 1, "0 - forwarded", 0 );
+    ui->cbCSSU->insertItem( 3, "2 - on hold", 2 );
+    ui->cbCSSU->insertItem( 4, "3 - retrieved", 3 );
+    ui->cbCSSU->insertItem( 5, "4 - multiparty", 4 );
+
+    ui->cbCSSI->insertItem( 0, "" );
+    ui->cbCSSI->insertItem( 3, "2 - forwarded", 2 );
+    ui->cbCSSI->insertItem( 6, "5 - outgoing barred", 5 );
+    ui->cbCSSI->insertItem( 7, "6 - incoming barred", 6 );
+}
+
+void ControlWidget::sendCSSN()
+{
+    QVariant v = ui->cbCSSU->itemData( ui->cbCSSU->currentIndex() );
+
+    if ( v.canConvert<int>() && ui->cbCSSU->isEnabled() )
+        emit unsolicitedCommand( "+CSSU: "+QString::number( v.toInt() ) );
+
+    v = ui->cbCSSI->itemData( ui->cbCSSI->currentIndex() );
+
+    if ( v.canConvert<int>() && ui->cbCSSI->isEnabled() )
+        emit unsolicitedCommand( "+CSSI: "+QString::number( v.toInt() ) );
+}
+
 void ControlWidget::sendSQ()
 {
     emit variableChanged("SQ",QString::number(ui->hsSignalQuality->value())+",99");
     emit unsolicitedCommand("+CSQ: "+QString::number(ui->hsSignalQuality->value())+",99");
 }
 
-
 void ControlWidget::sendBC()
 {
     bool charging = ui->hsBatteryCharging->checkState() == Qt::Checked;
@@ -221,6 +354,33 @@ void ControlWidget::sendSMSMessage()
     p->constructSMSMessage(ui->leSMSClass->text().toInt(), ui->leMessageSender->text(), ui->leSMSServiceCenter->text(), ui->teSMSText->toPlainText());
 }
 
+void ControlWidget::setDefaultGNSSData()
+{
+    ui->teGNSStext->clear();
+
+    ui->teGNSStext->append("<?xml version=\"1.0\" ?>");
+    ui->teGNSStext->append("<pos>");
+    ui->teGNSStext->append("  <pos_err>");
+    ui->teGNSStext->append("    <err_reason literal=\"not_enough_gps_satellites\" />");
+    ui->teGNSStext->append("  </pos_err>");
+    ui->teGNSStext->append("</pos>");
+}
+
+void ControlWidget::sendGNSSData()
+{
+    QStringList xml = ui->teGNSStext->toPlainText().split("\n");
+
+    foreach ( QString line, xml )
+    {
+        if (!line.isEmpty()) {
+            QString cposr =  "+CPOSR: " + line;
+
+            emit unsolicitedCommand(cposr);
+            handleFromData( cposr );
+        }
+    }
+}
+
 void ControlWidget::sendMGD()
 {
     emit command("AT+CMGD=1");
@@ -233,12 +393,19 @@ void ControlWidget::selectFile()
 
 void ControlWidget::sendSMSDatagram()
 {
-    QString portStr = ui->lePort->text();
-    if ( portStr.contains(QRegExp("\\D")) ) {
+    QString dstPortStr = ui->leDstPort->text();
+    if ( dstPortStr.contains(QRegExp("\\D")) ) {
         p->warning(tr("Invalid Port"), tr("Port number can contain only digits" ));
         return;
     }
-    int port = portStr.toInt();
+    int dst = dstPortStr.toInt();
+
+    QString srcPortStr = ui->leSrcPort->text();
+    if ( srcPortStr.contains(QRegExp("\\D")) ) {
+        p->warning(tr("Invalid Port"), tr("Port number can contain only digits" ));
+        return;
+    }
+    int src = srcPortStr.toInt();
 
     QString sender = ui->leDatagramSender->text();
 
@@ -268,12 +435,25 @@ void ControlWidget::sendSMSDatagram()
     }
 
     //construct and place SMS datagram in SMSList
-    p->constructSMSDatagram(port, sender, data, contentType);
+    p->constructSMSDatagram(src, dst, sender, data, contentType);
 }
 
 void ControlWidget::sendCall()
 {
-    emit startIncomingCall( ui->leCaller->text() );
+    QString number;
+    QString calledNumber;
+    QString name;
+
+    if (ui->cbCaller->isChecked())
+        number = ui->leCaller->text();
+
+    if (ui->cbCalledLine->isChecked())
+        calledNumber = ui->leCalledLine->text();
+
+    if (ui->cbCallerName->isChecked())
+        name = ui->leCallerName->text();
+
+    emit startIncomingCall( number, calledNumber, name );
 }
 
 void ControlWidget::handleFromData( const QString& cmd )
@@ -440,8 +620,7 @@ void ControlWidget::simInsertRemove()
 
 void ControlWidget::handleNewApp()
 {
-    ui->lblApplicationName->setText( "Current application: " +
-            p->getSimAppName() );
+    ui->cbSimApps->insertItems( 0, p->getSimAppsNameList() );
 }
 
 void Control::handleNewApp()
@@ -451,7 +630,8 @@ void Control::handleNewApp()
 
 void ControlWidget::simAppStart()
 {
-    p->simAppStart();
+    if ( ui->cbSimApps->currentIndex() >= 0 )
+        p->simAppStart( ui->cbSimApps->currentIndex() );
 }
 
 void ControlWidget::simAppAbort()
@@ -459,6 +639,11 @@ void ControlWidget::simAppAbort()
     p->simAppAbort();
 }
 
+void ControlWidget::modemSilentReset()
+{
+    emit unsolicitedCommand("+CRST:");
+}
+
 Script::Script(QObject *obj, Ui_ControlBase *ui) : QDBusAbstractAdaptor(obj)
 {
     /* Export tabs to be accessed by script */
@@ -479,6 +664,9 @@ Script::Script(QObject *obj, Ui_ControlBase *ui) : QDBusAbstractAdaptor(obj)
 
     QScriptValue qsTab6 = engine.newQObject(ui->tab_6);
     engine.globalObject().setProperty("tabSIM", qsTab6);
+
+    QScriptValue qsTab8 = engine.newQObject(ui->tab_8);
+    engine.globalObject().setProperty("tabPosition", qsTab8);
 }
 
 void Script::SetPath(const QString &path, const QDBusMessage &msg)
