@@ -50,6 +50,7 @@ protected slots:
                   QObject *target, const char *slot,
                   QSimCommand::ToPduOptions options
                         = QSimCommand::NoPduOptions );
+    void modemHandledCommand( const QSimCommand& cmd, int timeout );
     void controlEvent( const QSimControlEvent& event );
 
     virtual void mainMenu() = 0;
@@ -132,6 +133,8 @@ protected slots:
     void localInfoMenu( const QSimTerminalResponse& resp );
     void sendBIPMenu();
     void BIPMenu( const QSimTerminalResponse& resp );
+    void sendHandledMenu();
+    void handledMenuResp( const QSimTerminalResponse& resp );
 
 private:
     int sticksLeft;
